searchData={"items":[{"type":"module","title":"Arrows","doc":"A handful of (mostly) arrow macros with superpowers.","ref":"Arrows.html"},{"type":"module","title":"Installation - Arrows","doc":"The package can be installed by adding `arrows` to your list of dependencies in `mix.exs`: \n\n```elixir\ndef deps do\n  [\n    {:arrows, \"~> 0.2.0\"}\n  ]\nend\n```\n\nOr via git:\n```elixir\ndef deps do\n  [\n    {:arrows, git: \"https://github.com/bonfire-networks/arrows\", branch: \"main\"}\n  ]\nend\n```","ref":"Arrows.html#module-installation"},{"type":"module","title":"Documentation - Arrows","doc":"The Elixir [|> (\"pipe\") operator](https://hexdocs.pm/elixir/Kernel.html#%7C%3E/2) is one of the things that seems to get people excited about elixir. Probably in part because you then don't have to keep coming up with function names. Unfortunately it's kind of limiting. \nThe moment you need to pipe a parameter into a position that isn't the first one, it breaks down and you have to drop out of the pipeline format or write a secondary function to handle it.\n\nNot any more! By simply inserting `...` where you would like the value to be inserted, `Arrows` will override where it is placed. This allows you to keep on piping while accommodating that function with the annoying argument order. `Arrows` was inspired by [an existing library](https://hexdocs.pm/magritte/Magritte.html). \n\nHere is part of the test suite in lieu of examples:\n\n```elixir\ndefmodule ArrowsTest do\n  use ExUnit.Case\n  use Arrows\n\n  def double(x), do: x * 2\n  def double_fst(x, _), do: x * 2\n  def double_snd(_, x), do: x * 2\n  def add_snd_thd(_, x, y), do: x + y\n\n  test \"|>\" do\n    assert 4 == (2 |> double)\n    assert 4 == (2 |> double())\n    assert 4 == (2 |> double(...))\n    assert 8 == (2 |> double(double(...)))\n    assert 4 == (2 |> double_fst(1))\n    assert 4 == (2 |> double_fst(..., 1))\n    assert 8 == (2 |> double_fst(double(...), 1))\n    assert 4 == (2 |> double_snd(1, ...))\n    assert 8 == (2 |> double_snd(1, double(...)))\n    assert 3 == (2 |> add_snd_thd(1, ..., 1))\n    assert 4 == (2 |> add_snd_thd(1, ..., ...))\n    assert 6 == (2 |> add_snd_thd(1, ..., double(...)))\n    for x   {:ok, ...})\n    end\n  end\nend\n```\n\nA few little extra features you might notice here:\n* You can move the parameter into a subexpression, as in `2 |> double_fst(double(...), 1)` where\n  double will be called before the parameter is passed to `double_fst`.\n* You can use `...` multiple times, substituting it in multiple places.\n* The right hand side need not even be a function call, you can use any expression with `...`.\n\n#","ref":"Arrows.html#module-documentation"},{"type":"module","title":"Ok-pipe - Arrows","doc":"`Arrows` also provides an `ok-pipe` operator, `~>`, which only pipes into the next function if the result from the last one was considered a success. It's inspired by [OK](https://hexdocs.pm/ok/readme.html), but we have chosen to do things slightly differently so it better fits with our regular pipe.\n\ninput                    | result          |\n:----------------------- | :-------------- |\n`{:ok, x}`               | `fun.(x)`       |\n`{:error, e}`            | `{:error, e}`   |\n`nil`                    | `nil`           |\n`x when not is_nil(x)`   | `fun.(x)`       |\n\nIn the case of a function returning an ok/error tuple being on the left hand side, this is straightforward to determine. In the event of `{:ok, x}`, x will be passed into the right hand side to call. In the event of `{:error, x}`, the result will be `{:error, x}`.\n\nWe also deal with a lot of functions that indicate failure by returning nil. `~>` tries to 'do what I mean' for both of these so you can have one pipe operator to rule them all. If `nil` is a valid result, you must thus be sure to wrap it in an `ok` tuple when it occurs on the left hand side of `~>`.\n\n`|>` and `~>` compose in the way you'd expect; i.e. a `~>` receiving an error tuple or nil will stop executing the rest of the chain of (mixed) pipes.\n\n\nDocumentation can be found at [https://hexdocs.pm/arrows](https://hexdocs.pm/arrows).","ref":"Arrows.html#module-ok-pipe"},{"type":"macro","title":"Arrows.<~>/2","doc":"Like `||`, except with the logic applied by `~>`","ref":"Arrows.html#%3C~%3E/2"},{"type":"function","title":"Arrows.from_ok/1","doc":"","ref":"Arrows.html#from_ok/1"},{"type":"function","title":"Arrows.ok/1","doc":"","ref":"Arrows.html#ok/1"},{"type":"function","title":"Arrows.ok_or/2","doc":"","ref":"Arrows.html#ok_or/2"},{"type":"function","title":"Arrows.to_ok/1","doc":"","ref":"Arrows.html#to_ok/1"},{"type":"macro","title":"Arrows.|>/2","doc":"A more flexible drop-in replacement for the standard elixir pipe operator.\n\nSpecial features are unlocked when using the `...` (ellipsis) on the right hand side:\n\n* The right hand side need not be a function, it can be any expression containing the ellipsis.\n* The ellipsis will be replaced with the result of evaluating the hand side expression.\n* You may use the ellipsis multiple times and the left hand side will be calculated exactly once.\n\nYou can do crazy stuff with the ellipsis, but remember that people have to read it!","ref":"Arrows.html#%7C%3E/2"},{"type":"macro","title":"Arrows.|||/2","doc":"","ref":"Arrows.html#%7C%7C%7C/2"},{"type":"macro","title":"Arrows.~>/2","doc":"Like `OK.~>`","ref":"Arrows.html#~%3E/2"}],"content_type":"text/markdown"}